@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c 9651 2007-02-25 16:22:36Z nickm $ */
+/* $Id: connection.c 10801 2007-07-12 16:17:05Z nickm $ */
 const char connection_c_id[] =
-  "$Id: connection.c 9651 2007-02-25 16:22:36Z nickm $";
+  "$Id: connection.c 10801 2007-07-12 16:17:05Z nickm $";
 
 /**
  * \file connection.c
@@ -404,7 +404,7 @@ connection_about_to_close_connection(connection_t *conn)
   edge_connection_t *edge_conn;
   time_t now = time(NULL);
 
-  assert(conn->marked_for_close);
+  tor_assert(conn->marked_for_close);
 
   if (CONN_IS_EDGE(conn)) {
     if (!conn->edge_has_sent_end) {
