@@ -1,9 +1,9 @@
 /* Copyright (c) 2001-2004, Roger Dingledine.
- * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
+ * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: policies.c 8479 2006-09-24 17:20:41Z nickm $ */
+/* $Id: policies.c 9596 2007-02-16 20:39:37Z nickm $ */
 const char policies_c_id[] = \
-  "$Id: policies.c 8479 2006-09-24 17:20:41Z nickm $";
+  "$Id: policies.c 9596 2007-02-16 20:39:37Z nickm $";
 
 /**
  * \file policies.c
@@ -12,12 +12,19 @@ const char policies_c_id[] = \
 
 #include "or.h"
 
-static int expand_exit_policy_aliases(smartlist_t *entries, int assume_action);
-
+/** Policy that addresses for incoming SOCKS connections must match. */
 static addr_policy_t *socks_policy = NULL;
+/** Policy that addresses for incoming directory connections must match. */
 static addr_policy_t *dir_policy = NULL;
+/** Policy that addresses for incoming router descriptors must match in order
+ * to be published by us. */
 static addr_policy_t *authdir_reject_policy = NULL;
+/** Policy that addresses for incoming router descriptors must match in order
+ * to be marked as valid in our networkstatus. */
 static addr_policy_t *authdir_invalid_policy = NULL;
+/** Policy that addresses for incoming router descriptors must <b>not</b>
+ * match in order to not be marked as BadExit. */
+static addr_policy_t *authdir_badexit_policy = NULL;
 
 /** Parsed addr_policy_t describing which addresses we believe we can start
  * circuits at. */
@@ -51,10 +58,6 @@ parse_addr_policy(config_line_t *cfg, addr_policy_t **dest,
   for (; cfg; cfg = cfg->next) {
     smartlist_split_string(entries, cfg->value, ",",
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
-    if (expand_exit_policy_aliases(entries,assume_action)<0) {
-      r = -1;
-      continue;
-    }
     SMARTLIST_FOREACH(entries, const char *, ent,
     {
       log_debug(LD_CONFIG,"Adding new entry '%s'",ent);
@@ -128,7 +131,7 @@ parse_reachable_addresses(void)
 int
 firewall_is_fascist_or(void)
 {
-  return !!reachable_or_addr_policy;
+  return reachable_or_addr_policy != NULL;
 }
 
 /** Return true iff <b>policy</b> (possibly NULL) will allow a
@@ -153,6 +156,8 @@ addr_policy_permits_address(uint32_t addr, uint16_t port,
   }
 }
 
+/** Return true iff we think our firewall will let us make an OR connection to
+ * addr:port. */
 int
 fascist_firewall_allows_address_or(uint32_t addr, uint16_t port)
 {
@@ -160,6 +165,8 @@ fascist_firewall_allows_address_or(uint32_t addr, uint16_t port)
                                      reachable_or_addr_policy);
 }
 
+/** Return true iff we think our firewall will let us make a directory
+ * connection to addr:port. */
 int
 fascist_firewall_allows_address_dir(uint32_t addr, uint16_t port)
 {
@@ -203,8 +210,21 @@ authdir_policy_valid_address(uint32_t addr, uint16_t port)
   return addr_policy_permits_address(addr, port, authdir_invalid_policy);
 }
 
+/** Return 1 if <b>addr</b>:<b>port</b> should be marked as a bad exit,
+ * based on <b>authdir_badexit_policy</b>. Else return 0.
+ */
+int
+authdir_policy_badexit_address(uint32_t addr, uint16_t port)
+{
+  return ! addr_policy_permits_address(addr, port, authdir_badexit_policy);
+}
+
 #define REJECT(arg) \
   do { *msg = tor_strdup(arg); goto err; } while (0)
+
+/** Config helper: If there's any problem with the policy configuration
+ * options in <b>options</b>, return -1 and set <b>msg</b> to a newly
+ * allocated description of the error. Else return 0. */
 int
 validate_addr_policies(or_options_t *options, char **msg)
 {
@@ -243,7 +263,7 @@ err:
 #undef REJECT
 }
 
-/* Parse <b>string</b> in the same way that the exit policy
+/** Parse <b>string</b> in the same way that the exit policy
  * is parsed, and put the processed version in *<b>policy</b>.
  * Ignore port specifiers.
  */
@@ -262,6 +282,8 @@ load_policy_from_option(config_line_t *config, addr_policy_t **policy,
   }
 }
 
+/** Set all policies based on <b>options</b>, which should have been validated
+ * first. */
 void
 policies_parse_from_options(or_options_t *options)
 {
@@ -271,6 +293,8 @@ policies_parse_from_options(or_options_t *options)
                           &authdir_reject_policy, ADDR_POLICY_REJECT);
   load_policy_from_option(options->AuthDirInvalid,
                           &authdir_invalid_policy, ADDR_POLICY_REJECT);
+  load_policy_from_option(options->AuthDirBadExit,
+                          &authdir_badexit_policy, ADDR_POLICY_REJECT);
   parse_reachable_addresses();
 }
 
@@ -438,65 +462,6 @@ append_exit_policy_string(addr_policy_t **policy, const char *more)
   parse_addr_policy(&tmp, policy, -1);
 }
 
-static int
-expand_exit_policy_aliases(smartlist_t *entries, int assume_action)
-{
-  static const char *prefixes[] = {
-    "0.0.0.0/8", "169.254.0.0/16",
-    "127.0.0.0/8", "192.168.0.0/16", "10.0.0.0/8", "172.16.0.0/12",NULL };
-  int i;
-  char *pre=NULL, *post=NULL;
-  int expanded_any = 0;
-  pre = smartlist_join_strings(entries,",",0,NULL);
-  for (i = 0; i < smartlist_len(entries); ++i) {
-    char *v = smartlist_get(entries, i);
-    const char *cp, *ports;
-    const char *action;
-    int prefix_idx;
-    if (!strcasecmpstart(v, "accept")) {
-      action = "accept ";
-      cp = v+strlen("accept");
-    } else if (!strcasecmpstart(v, "reject")) {
-      action = "reject ";
-      cp = v+strlen("reject");
-    } else if (assume_action >= 0) {
-      action = "";
-      cp = v;
-    } else {
-      log_warn(LD_CONFIG,"Policy '%s' didn't start with accept or reject.", v);
-      tor_free(pre);
-      return -1;
-    }
-    cp = eat_whitespace(cp);
-    if (strcmpstart(cp, "private"))
-      continue; /* No need to expand. */
-    cp += strlen("private");
-    cp = eat_whitespace(cp);
-    if (*cp && *cp != ':')
-      continue; /* It wasn't "private" after all. */
-    ports = cp;
-    /* Okay. We're going to replace entries[i] with a bunch of new entries,
-     * in order. */
-    smartlist_del_keeporder(entries, i);
-    for (prefix_idx = 0; prefixes[prefix_idx]; ++prefix_idx) {
-      size_t replacement_len = 16+strlen(prefixes[prefix_idx])+strlen(ports);
-      char *replacement = tor_malloc(replacement_len);
-      tor_snprintf(replacement, replacement_len, "%s%s%s",
-                   action, prefixes[prefix_idx], ports);
-      smartlist_insert(entries, i++, replacement);
-    }
-    tor_free(v);
-    expanded_any = 1;
-    --i;
-  }
-  post = smartlist_join_strings(entries,",",0,NULL);
-  if (expanded_any)
-    log_info(LD_CONFIG, "Expanded '%s' to '%s'", pre, post);
-  tor_free(pre);
-  tor_free(post);
-  return expanded_any;
-}
-
 /** Detect and excise "dead code" from the policy *<b>dest</b>. */
 static void
 exit_policy_remove_redundancies(addr_policy_t **dest)
@@ -520,7 +485,7 @@ exit_policy_remove_redundancies(addr_policy_t **dest)
     tmp=ap;
     while (tmp) {
       if (tmp->next && addr_policy_covers(ap, tmp->next)) {
-        log(LOG_INFO, LD_CONFIG, "Removing exit policy %s.  It is made "
+        log(LOG_DEBUG, LD_CONFIG, "Removing exit policy %s.  It is made "
             "redundant by %s.", tmp->next->string, ap->string);
         victim = tmp->next;
         tmp->next = victim->next;
@@ -550,8 +515,8 @@ exit_policy_remove_redundancies(addr_policy_t **dest)
       }
       if (ap->policy_type == tmp->policy_type &&
           addr_policy_covers(tmp, ap)) {
-        log(LOG_INFO, LD_CONFIG, "Removing exit policy %s.  It is made "
-            "redundant by %s.", ap->string, tmp->string);
+        log(LOG_DEBUG, LD_CONFIG, "Removing exit policy %s.  It is already "
+            "covered by %s.", ap->string, tmp->string);
         victim = ap;
         ap = ap->next;
 
@@ -577,7 +542,8 @@ exit_policy_remove_redundancies(addr_policy_t **dest)
 
 #define DEFAULT_EXIT_POLICY                                         \
   "reject *:25,reject *:119,reject *:135-139,reject *:445,"         \
-  "reject *:465,reject *:587,reject *:1214,reject *:4661-4666,"     \
+  "reject *:465,reject *:563,reject *:587,"                         \
+  "reject *:1214,reject *:4661-4666,"                               \
   "reject *:6346-6429,reject *:6699,reject *:6881-6999,accept *:*"
 
 /** Parse the exit policy <b>cfg</b> into the linked list *<b>dest</b>. If
@@ -585,7 +551,6 @@ exit_policy_remove_redundancies(addr_policy_t **dest)
  * policy afterwards. If <b>rejectprivate</b> is true, prepend
  * "reject private:*" to the policy. Return -1 if we can't parse cfg,
  * else return 0.
- *
  */
 int
 policies_parse_exit_policy(config_line_t *cfg, addr_policy_t **dest,
@@ -603,11 +568,12 @@ policies_parse_exit_policy(config_line_t *cfg, addr_policy_t **dest,
 
 /** Return true iff <b>ri</b> is "useful as an exit node", meaning
  * it allows exit to at least one /8 address space for at least
- * one of ports 80, 443, and 6667. */
+ * two of ports 80, 443, and 6667. */
 int
 exit_policy_is_general_exit(addr_policy_t *policy)
 {
   static const int ports[] = { 80, 443, 6667 };
+  int n_allowed = 0;
   int i;
   for (i = 0; i < 3; ++i) {
     struct addr_policy_t *p = policy;
@@ -619,10 +585,104 @@ exit_policy_is_general_exit(addr_policy_t *policy)
       if ((p->addr & 0xff000000ul) == 0x7f000000ul)
         continue; /* 127.x */
       /* We have a match that is at least a /8. */
-      if (p->policy_type == ADDR_POLICY_ACCEPT)
-        return 1;
+      if (p->policy_type == ADDR_POLICY_ACCEPT) {
+        ++n_allowed;
+        break; /* stop considering this port */
+      }
     }
   }
+  return n_allowed >= 2;
+}
+
+/** Return false if <b>policy</b> might permit access to some addr:port;
+ * otherwise if we are certain it rejects everything, return true. */
+int
+policy_is_reject_star(addr_policy_t *p)
+{
+  for ( ; p; p = p->next) {
+    if (p->policy_type == ADDR_POLICY_ACCEPT)
+      return 0;
+    else if (p->policy_type == ADDR_POLICY_REJECT &&
+             p->prt_min <= 1 && p->prt_max == 65535 &&
+             p->msk == 0)
+      return 1;
+  }
+  return 1;
+}
+
+/** Write a single address policy to the buf_len byte buffer at buf.  Return
+ * the number of characters written, or -1 on failure. */
+int
+policy_write_item(char *buf, size_t buflen, addr_policy_t *policy)
+{
+  struct in_addr in;
+  size_t written = 0;
+  char addrbuf[INET_NTOA_BUF_LEN];
+  int result;
+
+  in.s_addr = htonl(policy->addr);
+  tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
+  /* write accept/reject 1.2.3.4 */
+  result = tor_snprintf(buf, buflen, "%s %s",
+            policy->policy_type == ADDR_POLICY_ACCEPT ? "accept" : "reject",
+            policy->msk == 0 ? "*" : addrbuf);
+  if (result < 0)
+    return -1;
+  written += strlen(buf);
+  /* If the mask is 0xffffffff, we don't need to give it.  If the mask is 0,
+   * we already wrote "*". */
+  if (policy->msk != 0xFFFFFFFFu && policy->msk != 0) {
+    int n_bits = addr_mask_get_bits(policy->msk);
+    if (n_bits >= 0) {
+      if (tor_snprintf(buf+written, buflen-written, "/%d", n_bits)<0)
+        return -1;
+    } else {
+      /* Write "/255.255.0.0" */
+      in.s_addr = htonl(policy->msk);
+      tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
+      if (tor_snprintf(buf+written, buflen-written, "/%s", addrbuf)<0)
+        return -1;
+    }
+    written += strlen(buf+written);
+  }
+  if (policy->prt_min <= 1 && policy->prt_max == 65535) {
+    /* There is no port set; write ":*" */
+    if (written+4 > buflen)
+      return -1;
+    strlcat(buf+written, ":*", buflen-written);
+    written += 2;
+  } else if (policy->prt_min == policy->prt_max) {
+    /* There is only one port; write ":80". */
+    result = tor_snprintf(buf+written, buflen-written, ":%d", policy->prt_min);
+    if (result<0)
+      return -1;
+    written += result;
+  } else {
+    /* There is a range of ports; write ":79-80". */
+    result = tor_snprintf(buf+written, buflen-written, ":%d-%d",
+                          policy->prt_min, policy->prt_max);
+    if (result<0)
+      return -1;
+    written += result;
+  }
+  if (written < buflen)
+    buf[written] = '\0';
+  else
+    return -1;
+
+  return (int)written;
+}
+
+/** Implementation for GETINFO control command: knows the answer for questions
+ * about "exit-policy/..." */
+int
+getinfo_helper_policies(control_connection_t *conn,
+                        const char *question, char **answer)
+{
+  (void) conn;
+  if (!strcmp(question, "exit-policy/default")) {
+    *answer = tor_strdup(DEFAULT_EXIT_POLICY);
+  }
   return 0;
 }
 
@@ -640,6 +700,7 @@ addr_policy_free(addr_policy_t *p)
   }
 }
 
+/** Release all storage held by policy variables. */
 void
 policies_free_all(void)
 {
