@@ -2,9 +2,9 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2007, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c 10301 2007-05-24 15:48:53Z nickm $ */
+/* $Id: routerparse.c 10846 2007-07-16 17:46:31Z nickm $ */
 const char routerparse_c_id[] =
-  "$Id: routerparse.c 10301 2007-05-24 15:48:53Z nickm $";
+  "$Id: routerparse.c 10846 2007-07-16 17:46:31Z nickm $";
 
 /**
  * \file routerparse.c
@@ -649,6 +649,8 @@ router_parse_list_from_string(const char **s, const char *eos,
 {
   routerinfo_t *router;
   const char *end, *cp, *start;
+  char *buf;
+  size_t buf_len;
 
   tor_assert(s);
   tor_assert(*s);
@@ -676,6 +678,11 @@ router_parse_list_from_string(const char **s, const char *eos,
       cp = end = eos;
     }
 
+    /* Start by backing up a character.  If we were at eos, we'll now point
+     * to a valid character. If we were at a \nrouter or \ndirectory-signature,
+     * we'll back up to before the \n. */
+    --cp;
+
     while (cp > *s && (!*cp || TOR_ISSPACE(*cp)))
       --cp;
     /* cp now points to the last non-space character in this descriptor. */
@@ -685,14 +692,23 @@ router_parse_list_from_string(const char **s, const char *eos,
     /* cp now points to the first \n before the last non-blank line in this
      * descriptor */
 
+    if (eos - cp < 25) /* not long enough to hold an "end signature" */
+      break;
+
     if (strcmpstart(cp, "\n-----END SIGNATURE-----\n")) {
       log_info(LD_DIR, "Ignoring truncated router descriptor.");
       *s = end;
       continue;
     }
 
-    router = router_parse_entry_from_string(*s, end,
+    /* router_parse_entry_from_string isn't necessarily safe if the string
+     * is non-NUL-terminated.  This fix is a workaround for the stable
+     * series only;  */
+    buf_len = end-*s;
+    buf = tor_strndup(*s, buf_len); /* nul-terminates the copy. */
+    router = router_parse_entry_from_string(buf, buf+buf_len,
                                             saved_location != SAVED_IN_CACHE);
+    tor_free(buf);
 
     if (!router) {
       log_warn(LD_DIR, "Error reading router; skipping");
@@ -948,7 +964,7 @@ router_parse_entry_from_string(const char *s, const char *end,
     goto err;
   }
   if (tok != find_last_by_keyword(tokens, K_ROUTER_SIGNATURE)) {
-    log_warn(LD_DIR, "Multiple signatures on one router. That's not ok.");
+    log_warn(LD_DIR, "Multiple signatures on one router. Ignoring.");
     goto err;
   }
   if (!tok->object_type ||
